# This stub file is necessary because `_scm_version.py`
# autogenerated on build and absent on mypy checks time
from typing import Tuple, Union


version: str
version_tuple: Tuple[Union[int, str], ...]
